@@ -55,12 +55,12 @@ class Alignment(object):
         return not self.__ne__(other)
 
     def __ne__(self, other):
-        if hash(self) == hash(other):
-            return False
-
-        if type(self) != type(other):
+        if not isinstance(self, other.__class__):
             return True
 
+        if getattr(other, "__hash__", None):
+            return hash(self) != hash(other)
+
         return self.offset != other.offset or self.grainSize != other.grainSize
 
     def __str__(self):
@@ -71,6 +71,9 @@ class Alignment(object):
               "ped": self.__alignment})
         return s
 
+    def __hash__(self):
+        return hash(str(self))
+
     @localeC
     def intersect(self, b):
         """Create and return a new Alignment that describes the intersection of
