@@ -25,8 +25,8 @@
 import _ped
 import parted
 
-from cachedlist import CachedList
-from decorators import localeC
+from parted.cachedlist import CachedList
+from parted.decorators import localeC
 
 class Disk(object):
     """Disk()
@@ -74,12 +74,12 @@ class Disk(object):
         return not self.__ne__(other)
 
     def __ne__(self, other):
-        if hash(self) == hash(other):
-            return False
-
-        if type(self) != type(other):
+        if not isinstance(self, other.__class__):
             return True
 
+        if getattr(other, "__hash__", None):
+            return hash(self) != hash(other)
+
         return self.device != other.device or not self._hasSameParts(other)
 
     def __str__(self):
@@ -95,6 +95,17 @@ class Disk(object):
               "ped": self.__disk})
         return s
 
+    @property
+    def _hash_str(self):
+        s = ("  type: %(type)s\n"
+             "  device: %(device)r\n"
+             "  PedDisk: %(ped)r" %
+             {"type": self.type, "device": self.device, "ped": self.__disk})
+        return s
+
+    def __hash__(self):
+        return hash(self._hash_str)
+
     def __getPartitions(self):
         """Construct a list of partitions on the disk.  This is called only as
            needed from the self.partitions property, which just happens to be
@@ -391,11 +402,12 @@ class Disk(object):
         """Return the extended Partition, if any, on this Disk."""
         try:
             return parted.Partition(disk=self, PedPartition=self.__disk.extended_partition())
+        # pylint: disable=bare-except
         except:
             return None
 
     def __filterPartitions(self, fn):
-        return filter(fn, self.partitions)
+        return [part for part in self.partitions if fn(part)]
 
     def getLogicalPartitions(self):
         """Return a list of logical Partitions on this Disk."""
@@ -473,7 +485,7 @@ while True:
     try:
         __type = _ped.disk_type_get_next(__type)
         diskType[__type.name] = __type
-    except:
+    except (IndexError, TypeError, _ped.UnknownTypeException):
         break
 
 # collect all disk flags and store them in a hash
