@@ -25,7 +25,7 @@
 import _ped
 import parted
 
-from decorators import localeC
+from parted.decorators import localeC
 
 # XXX: add docstrings!
 
@@ -62,12 +62,12 @@ class FileSystem(object):
         return not self.__ne__(other)
 
     def __ne__(self, other):
-        if hash(self) == hash(other):
-            return False
-
-        if type(self) != type(other):
+        if not isinstance(self, other.__class__):
             return True
 
+        if getattr(other, "__hash__", None):
+            return hash(self) != hash(other)
+
         return self.type != other.type or self.geometry != other.geometry
 
     def __str__(self):
@@ -78,6 +78,17 @@ class FileSystem(object):
               "checked": self.checked, "ped": self.__fileSystem})
         return s
 
+    @property
+    def _hash_str(self):
+        s = ("  type: %(type)s  geometry: %(geometry)r\n"
+             "  PedFileSystem: %(ped)r" %
+             {"type": self.type, "geometry": self.geometry,
+              "ped": self.__fileSystem})
+        return s
+
+    def __hash__(self):
+        return hash(self._hash_str)
+
     @property
     def type(self):
         """The type of this filesystem, e.g. ext3."""
@@ -107,5 +118,5 @@ while True:
     try:
         __type = _ped.file_system_type_get_next(__type)
         fileSystemType[__type.name] = __type
-    except:
+    except (IndexError, TypeError, _ped.UnknownTypeException):
         break
